/**********************************************************************
* FILENAME
*   block.c
*
* DESCRIPTION
*   Provides elementary functions which are necessary to implement 
*   arbitrary signal flow graphs in a block processing environment
*
* Rev 1.2  Dec.7, 2001 		C. Feldbauer <feldbauer@inw.tugraz.at>
**********************************************************************/


#include "block.h"


void block_float_copy( float *dst, float *src, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    dst[i] = src[i];
}

void block_short_copy( short *dst, short *src, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    dst[i] = src[i];
}




void block_float_add(float *dst, float *src, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    dst[i] += src[i];
}

void block_short_add(short *dst, short *src, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    dst[i] += src[i];
}




void block_float_sub( float *a, float *b, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    a[i] = a[i] - b[i];
}

void block_short_sub( short *a, short *b, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    a[i] = a[i] - b[i];
}


void block_float_c_is_a_minus_b( float *c, float *a, float *b, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    c[i] = a[i] - b[i];
}

void block_short_c_is_a_minus_b( short *c, short *a, short *b, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    c[i] = a[i] - b[i];
}




void block_float_multiply( float *dst, float *src, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    dst[i] *= src[i];
}

void block_short_multiply_q15( short *dst, short *src, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    dst[i] = (short)(_mpy( dst[i], src[i]) >> 15);
}




void block_float_multiply_const( float *sig, float constant, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    sig[i] *= constant;
}

void block_short_multiply_const_q15( short *sig, short constant, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    sig[i] = (short)(_mpy( sig[i], constant) >> 15);
}




void block_float_negate( float *sig, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    sig[i] = -sig[i];
}

void block_short_negate( short *sig, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    sig[i] = -sig[i];
}




void block_float2short( float *floatbuf, short *shortbuf, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    shortbuf[i] = (short)floatbuf[i];
}

void block_short2float( short *shortbuf, float *floatbuf, int blksz)
{
  int i;
  for( i=0; i<blksz; i++)
    floatbuf[i] = (float)shortbuf[i];
}




void block_float_decimate( float *dst, float *src, int dec_fact, int blksz)
{
  int i, j=0;
  for( i=0; i<blksz; i+=dec_fact)
    dst[j++] = src[i];
}

void block_short_decimate( short *dst, short *src, int dec_fact, int blksz)
{
  int i, j=0;
  for( i=0; i<blksz; i+=dec_fact)
    dst[j++] = src[i];
}




void block_float_decimate_off( float *dst, float *src, int dec_fact, int off, int blksz)
{
  int i, j=0;
  for( i=off; i<blksz; i+=dec_fact)
    dst[j++] = src[i];
}

void block_short_decimate_off( short *dst, short *src, int dec_fact, int off, int blksz)
{
  int i, j=0;
  for( i=off; i<blksz; i+=dec_fact)
    dst[j++] = src[i];
}




void block_float_interpolate( float *dst, float *src, int int_fact, int blksz)
{
  int i, j, k;  
  j = int_fact*blksz-1;
  for( i=blksz-1; i>=0; i--)
  {
    for( k=1; k<int_fact; k++)
      dst[j--] = 0.0;
    dst[j--] = src[i];
  }
}

void block_short_interpolate( short *dst, short *src, int int_fact, int blksz)
{
  int i, j, k;
  j = int_fact*blksz-1;
  for( i=blksz-1; i>=0; i--)
  {
    for( k=1; k<int_fact; k++)
      dst[j--] = 0;
    dst[j--] = src[i];
  }
}




void block_float_interpolate_off( float *dst, float *src, int int_fact, int off, int blksz)
{
  int i, j, k;
  j = int_fact*blksz-1;
  for( i=blksz-1; i>=0; i--)
  {
    for( k=int_fact-1; k>=0; k--)
    {
      if(k==off)
        dst[j--] = src[i];
      else
        dst[j--] = 0.0;
    }
  }
}

void block_short_interpolate_off( short *dst, short *src, int int_fact, int off, int blksz)
{
  int i, j, k;
  j = int_fact*blksz-1;
  for( i=blksz-1; i>=0; i--)
  {
    for( k=int_fact-1; k>=0; k--)
    {
      if(k==off)
        dst[j--] = src[i];
      else
        dst[j--] = 0;
    }
  }
}
